"""
Contains the methods to check compliance for Load Balancers
"""

import logging

from botocore.exceptions import ClientError

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# Checks the compliance for alb-http-drop-invalid-header-enabled
def alb_http_drop_invalid_header_enabled(self, **kwargs) -> dict:
    """
    :param self:
    :param kwargs:
    :return:
    """
    logger.info(" ---Inside lb :: alb_http_drop_invalid_header_enabled()--- ")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id3.4'
    compliance_type = 'ALB HTTP drop invalid header enabled'
    description = "Checks if rule evaluates AWS Application Load Balancers (ALB) to ensure they are configured to " \
                  "drop http headers. The rule is NON_COMPLIANT if the value of " \
                  "routing.http.drop_invalid_header_fields.enabled is set to false"
    resource_type = 'ELB'
    risk_level = 'Medium'

    if 'exception' in kwargs.keys() and kwargs['exception']:
        return {
            'Result': False,
            'failReason': kwargs['exception_text'],
            'resource_type': resource_type,
            'Offenders': offenders,
            'Compliance_type': compliance_type,
            'Description': description,
            'Risk Level': risk_level,
            'ControlId': control_id
        }

    for region, lb_list in kwargs['lb_list'].items():
        client = self.session.client('elbv2', region_name=region)
        for lb in lb_list:
            if lb['Type'] == 'application':
                response = client.describe_load_balancer_attributes(
                    LoadBalancerArn=lb['LoadBalancerArn']
                )
                for attr in response['Attributes']:
                    if attr['Key'] == 'routing.http.drop_invalid_header_fields.enabled' and attr['Value'] == 'false':
                        result = False
                        failReason = 'ALB http drop invalid header fields is not enabled'
                        offenders.append(lb['LoadBalancerName'])

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level,
        'ControlId': control_id
    }


# checks the compliance for alb-waf-enabled
def alb_waf_enabled(self, **kwargs) -> dict:
    """
    :param self:
    :param kwargs:
    :return:
    """
    logger.info(" ---Inside lb :: alb_waf_enabled()--- ")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id3.6'
    compliance_type = 'ALB WAF Enabled'
    description = "Checks if Web Application Firewall (WAF) is enabled on Application Load Balancers (ALBs). " \
                  "This rule is NON_COMPLIANT if key: waf.enabled is set to false"
    resource_type = 'ELB'
    risk_level = 'Medium'

    if 'exception' in kwargs.keys() and kwargs['exception']:
        return {
            'Result': False,
            'failReason': kwargs['exception_text'],
            'resource_type': resource_type,
            'Offenders': offenders,
            'Compliance_type': compliance_type,
            'Description': description,
            'Risk Level': risk_level,
            'ControlId': control_id
        }

    for region, lb_list in kwargs['lb_list'].items():
        client = self.session.client('elbv2', region_name=region)
        for lb in lb_list:
            if lb['Type'] == 'application':
                response = client.describe_load_balancer_attributes(
                    LoadBalancerArn=lb['LoadBalancerArn']
                )
                found = False
                for attr in response['Attributes']:
                    if attr['Key'] == 'waf.enabled':
                        found = True
                        if attr['Value'] == 'false':
                            result = False
                            failReason = 'ALB WAF is not enabled'
                            offenders.append(lb['LoadBalancerName'])
                        break

                if not found:
                    result = False
                    failReason = 'ALB WAF is not enabled'
                    offenders.append(lb['LoadBalancerName'])

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level,
        'ControlId': control_id
    }


# checks the compliance for elbv2-multiple-az
def elbv2_multiple_az(self, **kwargs) -> dict:
    """
    :param self:
    :param kwargs:
    :return:
    """
    logger.info(" ---Inside lb :: elbv2_multiple_az()--- ")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id30.4'
    compliance_type = 'ELBV2 Multiple AZ'
    description = "Checks if an Elastic Load Balancer V2 (Application, Network, or Gateway Load Balancer) has " \
                  "registered instances from multiple Availability Zones (AZ's). The rule is NON_COMPLIANT if an " \
                  "Elastic Load Balancer V2 has instances registered in less than 2 AZ's"
    resource_type = 'ELB'
    risk_level = 'Medium'

    if 'exception' in kwargs.keys() and kwargs['exception']:
        return {
            'Result': False,
            'failReason': kwargs['exception_text'],
            'resource_type': resource_type,
            'Offenders': offenders,
            'Compliance_type': compliance_type,
            'Description': description,
            'Risk Level': risk_level,
            'ControlId': control_id
        }

    for region, lb_list in kwargs['lb_list'].items():
        for lb in lb_list:
            if len(lb['AvailabilityZones']) <= 1:
                result = False
                failReason = "Elastic Load Balancer V2 has instances registered in less than 2 AZ's"
                offenders.append(lb['LoadBalancerName'])

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level,
        'ControlId': control_id
    }


# checks compliance for elbv2 acm certificate required
def elbv2_acm_certificate_required(self, **kwargs) -> dict:
    """
    :param self:
    :param kwargs:
    :return:
    """
    logger.info(" ---Inside lb :: elbv2_acm_certificate_required()---  ")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id3.51'
    compliance_type = 'ELBV2 ACM Certificate Required'
    description = "Checks if Application Load Balancers and Network Load Balancers have listeners that are " \
                  "configured to use certificates from AWS Certificate Manager (ACM). This rule is NON_COMPLIANT " \
                  "if at least 1 load balancer has at least 1 listener that is configured without a certificate " \
                  "from ACM or is configured with a certificate different from an ACM certificate"
    resource_type = 'ELB'
    risk_level = 'Medium'

    if 'exception' in kwargs.keys() and kwargs['exception']:
        return {
            'Result': False,
            'failReason': kwargs['exception_text'],
            'resource_type': resource_type,
            'Offenders': offenders,
            'Compliance_type': compliance_type,
            'Description': description,
            'Risk Level': risk_level,
            'ControlId': control_id
        }

    for region, lb_list in kwargs['lb_list'].items():
        client = self.session.client('elbv2', region_name=region)
        for lb in lb_list:
            if lb['Type'] in ['application', 'network']:
                marker = ''
                while True:
                    if marker == '':
                        response = client.describe_listeners(
                            LoadBalancerArn=lb['LoadBalancerArn']
                        )
                    else:
                        response = client.describe_listeners(
                            LoadBalancerArn=lb['LoadBalancerArn'],
                            Marker=marker
                        )
                    for listener in response['Listeners']:
                        for certificate in listener['Certificates']:
                            if not certificate['CertificateArn'].startswith("arn:aws:acm:"):
                                result = False
                                failReason = "Load Balancer listener is configured without certificate from ACM or " \
                                             "is configured with a certificate different from an ACM certificate"
                                offenders.append(lb['LoadBalancerArn'])

                    try:
                        marker = response['NextMarker']
                        if marker == '':
                            break
                    except KeyError:
                        break

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level,
        'ControlId': control_id
    }


# checks the compliance for elb-multiple-az
def clb_multiple_az(self, **kwargs) -> dict:
    """
    :param self:
    :param kwargs:
    :return:
    """
    logger.info(" ---Inside lb :: clb_multiple_az()--- ")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id56.3'
    compliance_type = 'CLB Multiple AZ'
    description = "Checks if a Classic Load Balancer spans multiple Availability Zones (AZs). The rule is " \
                  "NON_COMPLIANT if a Classic Load Balancer spans less than 2 AZs"
    resource_type = 'ELB'
    risk_level = 'Medium'

    if 'exception' in kwargs.keys() and kwargs['exception']:
        return {
            'Result': False,
            'failReason': kwargs['exception_text'],
            'resource_type': resource_type,
            'Offenders': offenders,
            'Compliance_type': compliance_type,
            'Description': description,
            'Risk Level': risk_level,
            'ControlId': control_id
        }

    for region, lb_list in kwargs['lb_list'].items():
        for lb in lb_list:
            if len(lb['AvailabilityZones']) <= 1:
                result = False
                failReason = "Classic Load Balancer has instances registered in less than 2 AZ's"
                offenders.append(lb['LoadBalancerName'])

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level,
        'ControlId': control_id
    }

#
# # Checks compliance for ELB predefined security policy ssl check
# def elb_predefined_security_policy_ssl_check(self, **kwargs) -> dict:
#     """
#     :param self:
#     :param kwargs:
#     :return:
#     """
#     logger.info(" ---Inside lb :: elb_predefined_security_policy_ssl_check()--- ")
#
#     result = True
#     failReason = ''
#     offenders = []
#     control_id = 'Id6.4'
#     compliance_type = 'ELB Predefined Security Policy SSL check'
#     description = "Checks whether your Classic Load Balancer SSL listeners are using a predefined policy. The rule " \
#                   "is only applicable if there are SSL listeners for the Classic Load Balancer"
#     resource_type = 'ELB'
#     risk_level = 'Medium'
#
#     if 'exception' in kwargs.keys() and kwargs['exception']:
#         return {
#             'Result': False,
#             'failReason': kwargs['exception_text'],
#             'resource_type': resource_type,
#             'Offenders': offenders,
#             'Compliance_type': compliance_type,
#             'Description': description,
#             'Risk Level': risk_level,
#             'ControlId': control_id
#         }
#
#     for region, lb_list in kwargs['lb_list'].items():
#         client = self.session.client('elb', region_name=region)
#         for lb in lb_list:
#             try:
#                 response = client.describe_load_balancer_policies(
#                     LoadBalancerName=lb['LoadBalancerName']
#                 )
#                 for desc in response['PolicyDescriptions']:
#                     for attr in desc['PolicyAttributeDescriptions']:
#                         if attr['AttributeName'] == 'Reference-Security-Policy' and attr['AttributeValue'] not in ['ELBSecurityPolicy-2016-08', 'ELBSecurityPolicy-TLS-1-2-2017-01', 'ELBSecurityPolicy-TLS-1-1-2017-01']:
#                             result = False
#                             offenders.append(lb['LoabBalancerName'])



import logging

import botocore

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


def get_regions(self):
    logger.info(" ---Inside utils :: get_regions()--- ")
    """Summary

    Returns:
        TYPE: Description
    """

    client = self.session.client('ec2', region_name='us-east-1')
    region_response = {}
    try:
        region_response = client.describe_regions()
    except botocore.exceptions.ClientError as error:
        if error.response['Error']['Code'] == 'AuthFailure':
            logger.error(f" AccessKey credentails not found here: {error}")
            exit(1)
    except botocore.exceptions.NoCredentialsError as e:
        logger.error(f" Unable to locate credentials: {e} ")
        exit(1)

    regions = [region['RegionName'] for region in region_response['Regions']]
    return regions


# return the list of rds clusters
def list_rds_clusters(self, regions: list) -> dict:
    """
    :param self:
    :param regions:
    :return:
    """
    logger.info(" ---Inside utils :: list_rds_clusters()--- ")

    cluster_lst = {}

    for region in regions:
        client = self.session.client('rds', region_name=region)
        marker = ''
        while True:
            if marker == '':
                response = client.describe_db_clusters()
            else:
                response = client.describe_db_clusters(
                    Marker=marker
                )
            cluster_lst.setdefault(region, []).extend(response['DBClusters'])

            try:
                marker = response['Marker']
                if marker == '':
                    break
            except KeyError:
                break

    return cluster_lst


# returns the list of ec2 instances
def list_ec2_instances(self, regions: list) -> dict:
    """
    :param self:
    :param regions:
    :return:
    """
    logger.info(" ---Inside utils :: list_ec2_instances()--- ")

    instances = {}

    for region in regions:
        client = self.session.client('ec2', region_name=region)
        marker = ''
        while True:
            if marker == '':
                response = client.describe_instances()
            else:
                response = client.describe_instances(
                    NextToken=marker
                )
            instances.setdefault(region, []).extend(response['Reservations'])

            try:
                marker = response['NextToken']
                if marker == '':
                    break
            except KeyError:
                break

    return instances


# returns the list of security Groups
def list_security_groups(self, regions: list) -> dict:
    """
    :param self:
    :param regions:
    :return:
    """
    logger.info(" ---Inside utils :: list_security_groups()--- ")

    sg_lst = {}

    for region in regions:
        client = self.session.client('ec2', region_name=region)
        marker = ''
        while True:
            if marker == '':
                response = client.describe_security_groups()
            else:
                response = client.describe_security_groups(
                    NextToken=marker
                )
            sg_lst.setdefault(region, []).extend(response['SecurityGroups'])

            try:
                marker = response['NextToken']
                if marker == '':
                    break
            except KeyError:
                break

    return sg_lst


# return the list of ENI
def list_eni(self, regions: list) -> dict:
    """
    :param self:
    :param regions:
    :return:
    """
    logger.info(" ---Inside utils :: list_eni()--- ")

    eni_lst = {}

    for region in regions:
        client = self.session.client('ec2', region_name=region)
        marker = ''
        while True:
            if marker == '':
                response = client.describe_network_interfaces()
            else:
                response = client.describe_network_interfaces(
                    NextToken=marker
                )
            eni_lst.setdefault(region, []).extend(response['NetworkInterfaces'])

            try:
                marker = response['NextToken']
                if marker == '':
                    break
            except KeyError:
                break

    return eni_lst


# returns the list of lambda functions
def list_lambda_functions(self, regions: list) -> dict:
    """
    :param self:
    :param regions:
    :return:
    """
    logger.info(" ---Inside utils :: list_lambda_functions()--- ")

    lambda_list = {}

    for region in regions:
        client = self.session.client('lambda', region_name=region)
        marker = ''
        while True:
            if marker == '':
                response = client.list_functions()
            else:
                response = client.list_functions(
                    Marker=marker
                )
            lambda_list.setdefault(region, []).extend(response['Functions'])

            try:
                if marker == '':
                    break
            except KeyError:
                break

    return lambda_list


# returns the list Elastic load balancers
def list_elbv2(self, regions: list) -> dict:
    """
    :param self:
    :param regions:
    :return:
    """
    logger.info(" ---Inside utils :: list_elbv2()--- ")

    elb_lst = {}

    for region in regions:
        client = self.session.client('elbv2', region_name=region)
        marker = ''
        while True:
            if marker == '':
                response = client.describe_load_balancers()
            else:
                response = client.describe_load_balancers(
                    Marker=marker
                )
            elb_lst.setdefault(region, []).extend(response['LoadBalancers'])

            try:
                marker = response['NextMarker']
                if marker == '':
                    break
            except KeyError:
                break

    return elb_lst


# returns the list of auto scaling groups
def list_auto_scaling_groups(self, regions: list) -> dict:
    """
    :param self:
    :param regions:
    :return:
    """
    logger.info(" ---Inside utils :: list_auto_scaling_groups()--- ")

    asg_lst = {}

    for region in regions:
        client = self.session.client('autoscaling', region_name=region)
        marker = ''
        while True:
            if marker == '':
                response = client.describe_auto_scaling_groups()
            else:
                response = client.describe_auto_scaling_groups(
                    NextToken=marker
                )

            asg_lst.setdefault(region, []).extend(response['AutoScalingGroups'])

            try:
                marker = response['NextToken']
                if marker == '':
                    break
            except KeyError:
                break

    return asg_lst


# returns the list of auto scaling launch configs
def list_as_launch_configs(self, regions: list) -> dict:
    """
    :param self:
    :param regions:
    :return:
    """
    logger.info(" ---Inside utils :: list_as_launch_configs()--- ")

    launch_configs = {}

    for region in regions:
        client = self.session.client('autoscaling', region_name=region)
        marker = ''
        while True:
            if marker == '':
                response = client.describe_launch_configurations()
            else:
                response = client.describe_launch_configurations(
                    NextToken=marker
                )
            launch_configs.setdefault(region, []).extend(response['LaunchConfigurations'])\

            try:
                marker = response['NextToken']
                if marker == '':
                    break
            except KeyError:
                break

    return launch_configs


# returns the list Elastic load balancers
def list_elb(self, regions: list) -> dict:
    """
    :param self:
    :param regions:
    :return:
    """
    logger.info(" ---Inside utils :: list_elb()--- ")

    elb_lst = {}

    for region in regions:
        client = self.session.client('elb', region_name=region)
        marker = ''
        while True:
            if marker == '':
                response = client.describe_load_balancers()
            else:
                response = client.describe_load_balancers(
                    Marker=marker
                )
            elb_lst.setdefault(region, []).extend(response['LoadBalancerDescriptions'])

            try:
                marker = response['NextMarker']
                if marker == '':
                    break
            except KeyError:
                break

    return elb_lst



"""
Contains the methods to check compliance for rds
"""

import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# checks compliance for RdsClusterIamAuthenticationEnabled
def rds_cluster_iam_authentication_enabled(self, **kwargs) -> dict:
    """
    :param self:
    :param kwargs:
    :return:
    """
    logger.info(" ---Inside rds :: rds_cluster_iam_authentication_enabled()--- ")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id95.1'
    compliance_type = 'Rds Cluster Iam Authentication Enabled'
    description = "Checks if an Amazon RDS Cluster has AWS Identity and Access Management (IAM) authentication enabled"
    resource_type = 'RDS'
    risk_level = 'Medium'

    if 'exception' in kwargs.keys() and kwargs['exception']:
        return {
            'Result': False,
            'failReason': kwargs['exception_text'],
            'resource_type': resource_type,
            'Offenders': offenders,
            'Compliance_type': compliance_type,
            'Description': description,
            'Risk Level': risk_level,
            'ControlId': control_id
        }

    for region, clusters in kwargs['cluster_list'].items():
        for cluster in clusters:
            if not cluster['IAMDatabaseAuthenticationEnabled']:
                result = False
                failReason = 'IAM Database Authentication is not enabled'
                offenders.append(cluster['DBClusterIdentifier'])

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level,
        'ControlId': control_id
    }


from boto3 import session
import logging

from botocore.exceptions import ClientError

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()

__author__ = 'Klera DevOps'
__version__ = '0.0.1'


class aws_client:
    def __init__(self, **kwargs):
        if 'aws_access_key_id' in kwargs.keys() and 'aws_secret_access_key' in kwargs.keys():
            self.session = session.Session(
                aws_access_key_id=kwargs['aws_access_key_id'],
                aws_secret_access_key=kwargs['aws_secret_access_key'],
            )
        elif 'profile_name' in kwargs.keys():
            self.session = session.Session(profile_name=kwargs['profile_name'])

    from .utils import get_regions, list_rds_clusters, list_ec2_instances, list_eni, list_security_groups, \
        list_lambda_functions, list_elbv2, list_auto_scaling_groups, list_as_launch_configs, list_elb
    from .rds import rds_cluster_iam_authentication_enabled
    from .ec2 import ec2_token_hop_limit_check, ec2_security_group_attached_to_eni, ec2_instance_role_assigned
    from .s3 import alarm_s3_bucket_policy_change, s3_lifecycle_policy_check
    from .lambda_fn import lambda_function_settings_check
    from .lb import alb_http_drop_invalid_header_enabled, alb_waf_enabled, elbv2_multiple_az, \
        elbv2_acm_certificate_required, clb_multiple_az
    from .auto_scaling import auto_scaling_multiple_az, auto_scaling_launch_config_requires_imdsv2, \
        auto_scaling_launch_config_hop_limit

    def get_compliance(self) -> list:
        """
        :return:
        """
        regions = self.get_regions()

        compliance = []

        try:
            rds_clusters = self.list_rds_clusters(regions=regions)
        except ClientError as e:
            compliance.append(self.rds_cluster_iam_authentication_enabled(
                exception=True, exception_text=e.response['Error']['Code'])
            )
        else:
            compliance.append(self.rds_cluster_iam_authentication_enabled(cluster_list=rds_clusters))

        try:
            ec2_instances = self.list_ec2_instances(regions=regions)
        except ClientError as e:
            compliance.append(self.ec2_token_hop_limit_check(
                exception=True, exception_text=e.response['Error']['Code'])
            )
        else:
            compliance.append(self.ec2_token_hop_limit_check(ec2_instances=ec2_instances))
            compliance.append(self.ec2_instance_role_assigned(ec2_instances=ec2_instances))

        try:
            eni_lst = self.list_eni(regions=regions)
            sg_lst = self.list_security_groups(regions=regions)
        except ClientError as e:
            compliance.append(self.ec2_security_group_attached_to_eni(
                exception=True, exception_text=e.response['Error']['Code']))
        else:
            compliance.append(self.ec2_security_group_attached_to_eni(eni_lst=eni_lst, sg_lst=sg_lst))

        compliance.append(self.alarm_s3_bucket_policy_change(regions=regions))
        compliance.append(self.s3_lifecycle_policy_check())

        try:
            lambda_list = self.list_lambda_functions(regions=regions)
        except ClientError as e:
            compliance.append(self.lambda_function_settings_check(
                exception=True, exception_text=e.response['Error']['Code']
            ))
        else:
            compliance.append(self.lambda_function_settings_check(lambda_lst=lambda_list))

        try:
            elbv2_list = self.list_elbv2(regions=regions)
        except ClientError as e:
            compliance.append(self.alb_http_drop_invalid_header_enabled(exception=True, exception_text=e.response['Error']['Code']))
            compliance.append(self.alb_waf_enabled(exception=True, exception_text=e.response['Error']['Code']))
            compliance.append(self.elbv2_multiple_az(exception=True, exception_text=e.response['Error']['Code']))
            compliance.append(self.elbv2_acm_certificate_required(exception=True, exception_text=e.response['Error']['Code']))
        else:
            compliance.append(self.alb_http_drop_invalid_header_enabled(lb_list=elbv2_list))
            compliance.append(self.alb_waf_enabled(lb_list=elbv2_list))
            compliance.append(self.elbv2_multiple_az(lb_list=elbv2_list))
            compliance.append(self.elbv2_acm_certificate_required(lb_list=elbv2_list))

        try:
            asg_lst = self.list_auto_scaling_groups(regions=regions)
        except ClientError as e:
            compliance.append(self.auto_scaling_multiple_az(
                exception=True, exception_text=e.response['Error']['Code'])
            )
        else:
            compliance.append(self.auto_scaling_multiple_az(asg_lst=asg_lst))

        try:
            launch_configs = self.list_as_launch_configs(regions=regions)
        except ClientError as e:
            compliance.append(self.auto_scaling_launch_config_requires_imdsv2(
                exception=True, exception_text=e.response['Error']['Code'])
            )
            compliance.append(self.auto_scaling_launch_config_hop_limit(
                exception=True, exception_text=e.response['Error']['Code'])
            )
        else:
            compliance.append(self.auto_scaling_launch_config_requires_imdsv2(launch_config_list=launch_configs))
            compliance.append(self.auto_scaling_launch_config_hop_limit(launch_config_list=launch_configs))

        try:
            elb_list = self.list_elb(regions=regions)
        except ClientError as e:
            compliance.append(self.clb_multiple_az(
                    exception=True, exception_text=e.response['Error']['Code'])
                )
        else:
            compliance.append(self.clb_multiple_az(lb_list=elb_list))

        return compliance



"""
Contains the methods to check compliance for ec2
"""

import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# Checks compliance for EC2 Token Hop limit check
def ec2_token_hop_limit_check(self, **kwargs) -> dict:
    """
    :param self:
    :param kwargs:
    :return:
    """
    logger.info(" ---Inside ec2 :: ec2_token_hop_limit_check--- ")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id45.2'
    compliance_type = 'EC2 Token hop limit check'
    description = "Checks if an Amazon Elastic Compute Cloud (EC2) instance metadata has a specified token hop limit " \
                  "that is equal to 1"
    resource_type = 'EC2'
    risk_level = 'Medium'

    if 'exception' in kwargs.keys() and kwargs['exception']:
        return {
            'Result': False,
            'failReason': kwargs['exception_text'],
            'resource_type': resource_type,
            'Offenders': offenders,
            'Compliance_type': compliance_type,
            'Description': description,
            'Risk Level': risk_level,
            'ControlId': control_id
        }

    for region, reservations in kwargs['ec2_instances'].items():
        for reservation in reservations:
            for instance in reservation['Instances']:
                try:
                    hop_limit = instance['MetadataOptions']['HttpPutResponseHopLimit']
                    if not hop_limit == 1:
                        result = False
                        failReason = "Hop limit is not set to 1"
                        offenders.append(instance['InstanceId'])
                except KeyError:
                    result = False
                    failReason = "Hop limit is not set to 1"
                    offenders.append(instance['InstanceId'])

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level,
        'ControlId': control_id
    }


# Checks compliance for Ec2 security group attached to eni
def ec2_security_group_attached_to_eni(self, **kwargs) -> dict:
    """
    :param self:
    :param kwargs:
    :return:
    """
    logger.info(" ---Inside ec2 :: ec2_security_group_attached_to_eni()--- ")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id18.2'
    compliance_type = 'EC2 Security Group Attached to ENI'
    description = "Checks if non-default security groups are attached to Elastic network interfaces (ENIs). The rule " \
                  "is NON_COMPLIANT if the security group is not associated with an elastic network interface (ENI)"
    resource_type = 'EC2'
    risk_level = 'Medium'

    if 'exception' in kwargs.keys() and kwargs['exception']:
        return {
            'Result': False,
            'failReason': kwargs['exception_text'],
            'resource_type': resource_type,
            'Offenders': offenders,
            'Compliance_type': compliance_type,
            'Description': description,
            'Risk Level': risk_level,
            'ControlId': control_id
        }

    associated_security_groups = []
    security_groups = []

    for region, eni_lst in kwargs['eni_lst'].items():
        for eni in eni_lst:
            for group in eni['Groups']:
                associated_security_groups.append(group['GroupId'])

    for region, sg_lst in kwargs['sg_lst'].items():
        for sg in sg_lst:
            if sg['GroupName'] != 'default':
                if not sg['GroupId'] in associated_security_groups:
                    result = False
                    failReason = 'EC2 Security Group Not Attached to ENI'
                    offenders.append(sg['GroupId'])

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level,
        'ControlId': control_id
    }


# Checks compliance for EC2 Instance Role Assigned
def ec2_instance_role_assigned(self, **kwargs) -> dict:
    """
    :param self:
    :param kwargs:
    :return:
    """
    logger.info(" ---Inside ec2 :: ec2_instance_role_assigned--- ")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id26.1'
    compliance_type = 'EC2 Instance Role Assigned'
    description = "Checks if an Amazon Elastic Compute Cloud (EC2) instance has role assigned"
    resource_type = 'EC2'
    risk_level = 'Medium'

    if 'exception' in kwargs.keys() and kwargs['exception']:
        return {
            'Result': False,
            'failReason': kwargs['exception_text'],
            'resource_type': resource_type,
            'Offenders': offenders,
            'Compliance_type': compliance_type,
            'Description': description,
            'Risk Level': risk_level,
            'ControlId': control_id
        }

    for region, reservations in kwargs['ec2_instances'].items():
        for reservation in reservations:
            for instance in reservation['Instances']:
                try:
                    role = instance['IamInstanceProfile']['Id']
                except KeyError:
                    result = False
                    failReason = "IAM role is not assigned to the EC2 instance"
                    offenders.append(instance['InstanceId'])

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level,
        'ControlId': control_id
    }



"""
Contains the methods to check compliance for auto scaling
"""

import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# Checks the compliance for auto-scaling multiple az
def auto_scaling_multiple_az(self, **kwargs) -> dict:
    """
    :param self:
    :param kwargs:
    :return:
    """
    logger.info(" ---Inside auto_scaling :: auto_scaling_multiple_az()--- ")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id85.4'
    compliance_type = 'Auto Scaling Multiple AZ'
    description = "Checks if the Auto Scaling group spans multiple Availability Zones. The rule is NON_COMPLIANT " \
                  "if the Auto Scaling group does not span multiple Availability Zones"
    resource_type = 'Auto Scaling'
    risk_level = 'Medium'

    if 'exception' in kwargs.keys() and kwargs['exception']:
        return {
            'Result': False,
            'failReason': kwargs['exception_text'],
            'resource_type': resource_type,
            'Offenders': offenders,
            'Compliance_type': compliance_type,
            'Description': description,
            'Risk Level': risk_level,
            'ControlId': control_id
        }

    for region, asg_lst in kwargs['asg_lst'].items():
        for asg in asg_lst:
            if len(asg['AvailabilityZones']) <= 1:
                result = False
                failReason = 'AWS Auto Scaling Group can launch EC2 instances only within a single Availability Zone'
                offenders.append(asg['AutoScalingGroupName'])

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level,
        'ControlId': control_id
    }


# checks compliance for Auto Scaling launch config requires imdsv2
def auto_scaling_launch_config_requires_imdsv2(self, **kwargs) -> dict:
    """
    :param self:
    :param kwargs:
    :return:
    """
    logger.info(" ---Inside auto_scaling :: auto_scaling_launch_config_requires_imdsv2()--- ")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id85.3'
    compliance_type = 'Auto Scaling Launch Configuration Requires IMDSV2'
    description = "Checks whether only IMDSv2 is enabled. This rule is NON_COMPLIANT if the " \
                  "Metadata version is not included in the launch configuration or if both Metadata " \
                  "V1 and V2 are enabled"
    resource_type = 'Auto Scaling'
    risk_level = 'Medium'

    if 'exception' in kwargs.keys() and kwargs['exception']:
        return {
            'Result': False,
            'failReason': kwargs['exception_text'],
            'resource_type': resource_type,
            'Offenders': offenders,
            'Compliance_type': compliance_type,
            'Description': description,
            'Risk Level': risk_level,
            'ControlId': control_id
        }

    for region, lc_list in kwargs['launch_config_list'].items():
        for lc in lc_list:
            try:
                httpTokens = lc['MetadataOptions']['HttpTokens']
            except KeyError:
                result = False
                failReason = 'Either metadata version 1 is used or configuration not found'
                offenders.append(lc['LaunchConfigurationName'])
            else:
                if httpTokens == 'optional':
                    result = False
                    failReason = 'Either metadata version 1 is used or configuration not found'
                    offenders.append(lc['LaunchConfigurationName'])

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level,
        'ControlId': control_id
    }


# checks compliance for Auto Scaling launch config hop limit
def auto_scaling_launch_config_hop_limit(self, **kwargs) -> dict:
    """
    :param self:
    :param kwargs:
    :return:
    """
    logger.info(" ---Inside auto_scaling :: auto_scaling_launch_config_hop_limit--- ")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id85.2'
    compliance_type = 'Auto Scaling Launch Configuration Hop Limit'
    description = "Checks the number of network hops that the metadata token can travel. This rule is NON_COMPLIANT " \
                  "if the Metadata response hop limit is greater than 1"
    resource_type = 'Auto Scaling'
    risk_level = 'Medium'

    if 'exception' in kwargs.keys() and kwargs['exception']:
        return {
            'Result': False,
            'failReason': kwargs['exception_text'],
            'resource_type': resource_type,
            'Offenders': offenders,
            'Compliance_type': compliance_type,
            'Description': description,
            'Risk Level': risk_level,
            'ControlId': control_id
        }

    for region, lc_list in kwargs['launch_config_list'].items():
        for lc in lc_list:
            try:
                hoplimit = lc['MetadataOptions']['HttpPutResponseHopLimit']
            except KeyError:
                result = False
                failReason = 'Either Hop limit is greater than 1 or configuration not found'
                offenders.append(lc['LaunchConfigurationName'])
            else:
                if hoplimit >1:
                    result = False
                    failReason = 'Either Hop limit is greater than 1 or configuration not found'
                    offenders.append(lc['LaunchConfigurationName'])

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level,
        'ControlId': control_id
    }


"""
Contains the methods to check compliance for s3
"""

import logging

from botocore.exceptions import ClientError

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# Checks compliance for Alarm s3 bucket policy change
def alarm_s3_bucket_policy_change(self, **kwargs) -> dict:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside s3 :: alarm_s3_bucket_policy_change()--- ")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id25.17'
    compliance_type = 'Alarm S3 Bucket Policy Change'
    description = "Ensure there is an Amazon CloudWatch alarm created and configured in your AWS account " \
                  "to fire each time a S3 bucket configuration change is made."
    resource_type = 'S3'
    risk_level = 'Medium'

    if 'exception' in kwargs.keys() and kwargs['exception']:
        return {
            'Result': False,
            'failReason': kwargs['exception_text'],
            'resource_type': resource_type,
            'Offenders': offenders,
            'Compliance_type': compliance_type,
            'Description': description,
            'Risk Level': risk_level,
            'ControlId': control_id
        }

    for region in kwargs['regions']:
        client = self.session.client('cloudwatch', region_name=region)
        response = client.describe_alarms_for_metric(
            MetricName='S3BucketEventCount',
            Namespace='CloudTrailMetrics'
        )
        if len(response['MetricAlarms']) == 0:
            result = False
            failReason = "Alarm does not exists for s3 bucket policy change"

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level,
        'ControlId': control_id
    }


# Checks compliance for s3 lifecycle policy check
def s3_lifecycle_policy_check(self) -> dict:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside s3 :: s3_lifecycle_policy_check()--- ")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id65.2'
    compliance_type = 'S3 lifecycle policy check'
    description = "Checks if a lifecycle rule is configured for an Amazon Simple Storage Service (Amazon S3) bucket. " \
                  "The rule is NON_COMPLIANT if there is no active lifecycle configuration rules or the " \
                  "configuration does not match with the parameter values."
    resource_type = 'S3'
    risk_level = 'Low'

    client = self.session.client('s3')
    buckets_response = client.list_buckets()

    for bucket in buckets_response['Buckets']:
        try:
            config_response = client.get_bucket_lifecycle_configuration(
                Bucket=bucket['Name']
            )
            flag = False
            for rule in config_response['Rules']:
                status = rule['Status']
                if status == 'Enabled':
                    flag = True
                    break

            if not flag:
                result = False
                offenders.append(bucket['Name'])
                failReason = "Either no lifecycle Configuration found, or set to disabled"

        except ClientError as e:
            if e.response['Error']['Code'] == 'NoSuchLifecycleConfiguration':
                result = False
                offenders.append(bucket['Name'])
                failReason = "Either no lifecycle Configuration found, or set to disabled"
            else:
                return {
                    'Result': False,
                    'failReason': e.response['Error']['Code'],
                    'resource_type': resource_type,
                    'Offenders': offenders,
                    'Compliance_type': compliance_type,
                    'Description': description,
                    'Risk Level': risk_level,
                    'ControlId': control_id
                }

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level,
        'ControlId': control_id
    }



"""
Contains the methods to check compliance for ec2
"""

import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# Checks the compliance for lambda-function-settings-check
def lambda_function_settings_check(self, **kwargs) -> dict:
    """
    :param self:
    :param kwargs:
    :return:
    """
    logger.info(" ---Inside lambda_fn :: lambda_function_settings_check()--- ")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id93.1'
    compliance_type = 'Lambda function settings check'
    description = "Checks if the AWS Lambda function settings for runtime, role, timeout, and memory size match " \
                  "the expected values. The rule ignores functions with the 'Image' package type"
    resource_type = 'Lambda'
    risk_level = 'Medium'

    if 'exception' in kwargs.keys() and kwargs['exception']:
        return {
            'Result': False,
            'failReason': kwargs['exception_text'],
            'resource_type': resource_type,
            'Offenders': offenders,
            'Compliance_type': compliance_type,
            'Description': description,
            'Risk Level': risk_level,
            'ControlId': control_id
        }

    for region, lambda_list in kwargs['lambda_lst'].items():
        for function in lambda_list:
            if not function['PackageType'] == 'Image':
                mem_size = function['MemorySize']
                timeout = function['Timeout']
                try:
                    role = function['Role']
                    if mem_size == 128 and timeout == 3:
                        pass
                    else:
                        result = False
                        failReason = 'Either memory size or timeout or role does not meet the expected values'
                        offenders.append(function['FunctionName'])

                except KeyError:
                    result = False
                    failReason = 'Either memory size or timeout or role does not meet the expected values'
                    offenders.append(function['FunctionName'])

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level,
        'ControlId': control_id
    }